#include "CoMSSBasedSolver.h"

CoMSSBasedSolver::CoMSSBasedSolver(TaskType task, map<string,string> *additionalParams) {
  this->task = task;
  this->additionalParam = (*additionalParams)["-a"];
}

CoMSSBasedSolver::~CoMSSBasedSolver(){}

void CoMSSBasedSolver::computeProblem() {
  result = "UNDEFINED";
  vector<int> *assumps = new vector<int>;
  switch(task) {
  case TASK_ONE_EXT:
    computeOneMaximalExtension(assumps);
    delete assumps;
    return;
  case TASK_ALL_EXTS:
    computeAllExtensions(assumps);
    delete assumps;
    return;
  case TASK_CRED_INF:
    if(!additionalParam.compare("")) return;
    assumps->push_back(varMap->getVar(additionalParam)-1);
    computeOneExtension(assumps, false);
    delete assumps;
    return;
  case TASK_SKEP_INF:
    delete assumps;
    cerr << "Preferred Skeptical Inference Not Implemented Yet" << endl ;
    exit(0);
  default:
    return;
  }
}

void CoMSSBasedSolver::computeOneExtension(vector<int> *assumps, bool resultIsModel) {
  bool ret = hasAModel(assumps);
  if (ret) {
    result = resultIsModel ? modelToStringExtension(getModel()) : "YES";
  } else {
    result = "NO";
  }
}

string CoMSSBasedSolver::modelToStringExtension(vector<bool> *model) {
  string res;

  bool noneFoundYet = true;
  res = "[";
  for (int i = 0; i < nbArgs; i++) {
    if ((*model)[i]) {
      if(noneFoundYet) noneFoundYet=false; else res=res + ",";
      res = res + varMap->getName(i+1);
    }
  }
  res = res + "]";
  return res;
}

void CoMSSBasedSolver::displaySolution(ostream *out){
  *out << result << endl;
}
