#include "CompleteSemanticProblemSolver.h"


using namespace std;


CompleteSemanticProblemSolver::CompleteSemanticProblemSolver(TaskType task, map<string,string> *additionalParams) : DefaultSatBasedSolver(task, additionalParams) {}

CompleteSemanticProblemSolver::~CompleteSemanticProblemSolver() {}


// [bigwedge_{a \in A}(( a => -Pa) wedge (a <=> \bigwedge_{(b,a) in R}(Pb)))]
// wedge [bigwedge_{a:(b,a) in R}(Pa <=> (bigvee_{(b,a) in R} b))]
// 1,...,n <-> a_1,...,a_n
// n+1,...,2n <-> Pa_1,...,Pa_n


void CompleteSemanticProblemSolver::initProblem(Attacks *attacks, VarMap *varMap){
  this->varMap = varMap;
  vector<int> *vars = varMap->intVars();
  nbArgs = vars->size();
  for(int i=0; i<2*nbArgs; solver.newVar(), ++i); // init solver vars

  vector<int> *attacksToCurrentVar;
  Minisat::vec<Minisat::Lit> binaryClause, completeClause, oppositesClause, binaryOppositesClause, completeOppositesClause;

  for(vector<int>::iterator itVars = vars->begin() ; itVars != vars->end(); ++itVars) { // iterate over vars representing arguments
    oppositesClause.push(~Minisat::mkLit(*itVars-1)); // (-a vee -Pa)
    oppositesClause.push(~Minisat::mkLit(*itVars-1+nbArgs));
    solver.addClause_(oppositesClause, true);
    oppositesClause.clear();

    completeClause.push(Minisat::mkLit(*itVars-1));
    completeOppositesClause.push(~Minisat::mkLit(*itVars-1+nbArgs));
    attacksToCurrentVar = attacks->getAttacksTo(*itVars);
    if(attacksToCurrentVar->size() == 0) {
      solver.addClause_(completeClause, true);
      solver.addClause_(completeOppositesClause, true);
      completeClause.clear();
      completeOppositesClause.clear();
      continue;
    }

    for(vector<int>::iterator itAttackers = attacksToCurrentVar->begin(); itAttackers != attacksToCurrentVar->end(); ++itAttackers) {
      binaryOppositesClause.push(Minisat::mkLit(*itVars-1+nbArgs)); // Pa vee -b
      binaryOppositesClause.push(~Minisat::mkLit(*itAttackers-1));
      solver.addClause_(binaryOppositesClause, true);
      binaryOppositesClause.clear();
      completeOppositesClause.push(Minisat::mkLit(*itAttackers-1));

      binaryClause.push(~Minisat::mkLit(*itVars-1)); // -a vee Pb
      binaryClause.push(Minisat::mkLit(*itAttackers-1+nbArgs));
      solver.addClause_(binaryClause, true);
      binaryClause.clear();
      completeClause.push(~Minisat::mkLit(*itAttackers-1+nbArgs));
    }
    solver.addClause_(completeClause, true);
    solver.addClause_(completeOppositesClause, true);
    completeClause.clear();
    completeOppositesClause.clear();
  }
  delete vars;
  solver.useAsCompleteSolver();
}
