#ifndef __COMPLETE_SEMANTIC_PROBLEM_SOLVER_H__
#define __COMPLETE_SEMANTIC_PROBLEM_SOLVER_H__

#include "DefaultSatBasedSolver.h"


using namespace std;


/** 
 * \class CompleteSemanticProblemSolver
 * \brief a minisat based solver for complete semantic tasks
 */
class CompleteSemanticProblemSolver : public DefaultSatBasedSolver {
  
 public :
  
  /**
   * \fn CompleteSemanticProblemSolver(TaskType,string)
   * \brief initialize the problem solver given a task
   * \param task : the task (SE, EE, DC, DS) that is asked
   * \param additionalParams the additional parameters from the command line
   */
  CompleteSemanticProblemSolver(TaskType task, map<string,string> *additionalParams);

  /**
   * \fn CompleteSemanticProblemSolver::initProblem(Attacks*,VarMap*)
   * \brief initialize a problem given attack relations and variable mappings
   * \param attacks : the attacks relation
   * \param varMap : the variable mappings (name <-> solverVariable)
   */
  void initProblem(Attacks *attacks, VarMap *varMap);

  /**
   * \fn ~CompleteSemanticProblemSolver
   * \brief destructor
   */
  ~CompleteSemanticProblemSolver();

  
};


#endif
