#include "DefaultSatBasedSolver.h"


using namespace std;


DefaultSatBasedSolver::DefaultSatBasedSolver(TaskType task, map<string,string> *additionalParams) : SatBasedSolver(task, additionalParams) {}

DefaultSatBasedSolver::~DefaultSatBasedSolver() {}


void DefaultSatBasedSolver::displayDimacs(){
  Minisat::vec<Minisat::Lit> assumps;
  switch(task) {
  case TASK_CRED_INF:
    if(!additionalParam.compare("")) return;
    assumps.push(Minisat::mkLit(varMap->getVar(additionalParam)-1));
  case TASK_SKEP_INF:
    if(!additionalParam.compare("")) return;
    assumps.push(~Minisat::mkLit(varMap->getVar(additionalParam)-1));
  default:
    ;
  }
  solver.toDimacs(stdout, assumps);
}


vector<bool>* DefaultSatBasedSolver::getModel() {
  vector<bool> *res = new vector<bool>;
  for (int i = 0; i < nbArgs; i++) {
    res->push_back(solver.model[i] == MINISAT_LBOOL_TRUE ? true : false);
  }
  return res;
}

bool DefaultSatBasedSolver::hasAModel(vector<int> *assumps) {
  Minisat::vec<Minisat::Lit> lits;
  for(vector<int>::iterator itVars = assumps->begin(); itVars != assumps->end(); ++itVars) {
    lits.push(*itVars > 0 ? Minisat::mkLit(*itVars-1) : ~Minisat::mkLit(-*itVars-1));
  }
  Minisat::lbool ret = solver.solveLimited(lits);
  if(ret == MINISAT_LBOOL_TRUE)
    return true;
  else if (ret == MINISAT_LBOOL_FALSE)
    return false;
  else {
    cerr << "ERR:: Minisat returned LBOOL_UNDEF" << endl;
    return false;
  }
}

void DefaultSatBasedSolver::addBlockingClause() {
  Minisat::vec<Minisat::Lit> blockingClause;
  for (int i = 0; i < solver.nVars(); i++) {
    if (solver.model[i] != MINISAT_LBOOL_UNDEF) {
      blockingClause.push(solver.model[i] == MINISAT_LBOOL_TRUE ? ~Minisat::mkLit(i): Minisat::mkLit(i));
    }
  }
  solver.addClause_(blockingClause);
}
