#ifndef __EXTERNAL_COMSS_BASED_SOLVER_H__
#define __EXTERNAL_COMSS_BASED_SOLVER_H__

#include <iostream>
#include <string>
#include <fstream>
#include <sstream>
#include <algorithm>

#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <cstring>

#include "CoMSSBasedSolver.h"


class ExternalCoMSSBasedSolver : public CoMSSBasedSolver {

 public:
  /**
   * \fn ExternalCoMSSBasedSolver
   * \brief Constructor
   * \param semName : the accpetability semantics of the argument
   * \param task : the task (SE, EE, DC, DS) that is asked
   * \param commandline : the commandline used to call the external solver ; must contain FILE (replaced by a DIMACS WCNF temporaly source file by CoQuiAAS at execution time)
   * \param additionalParams the additional parameters from the command line
   */
  ExternalCoMSSBasedSolver(SemanticName semName, TaskType task, string commandline, map<string,string> *additionalParams);
  void initProblem(Attacks *attacks, VarMap *varMap);

 private:
  /**
   * \fn launchExternalSolver
   * \brief Used to call an external software as the CoMSS solver
   * \param cmdLine the command line used to call the external solver
   * \param file the temporary file which contains the WCNF instance to be solved by the external solver
   */
  void launchExternalSolver(string cmdLine, string file);
  
  /**
   * \fn addClause(cl,isHard)
   * \brief Add a clause (hard or soft) to the problem
   */
  void addClause(vector<int> *cl, bool isHard);

  void computeOneMaximalExtension(vector<int> *assumps);
  void computeAllExtensions(vector<int> *assumps);

  void addMSSBlockingClause();

    /** The semantics */
  SemanticName semName;

  /** The command line to launch the external solver */
  string commandline;

  /** Buffer to write the SAT instance corresponding to the encoding */
  stringstream strDimacs;

  /** The number of constraints in the SAT instance */
  unsigned int ncstrs;

  /** Must be true if the problem is satisfiable */
  bool isSat;

  /** Weight of hard clauses */
  const static int WCNF_TOP = (1<<30);

  /** Vector of boolean variables to represent a MSS of the problem */
  vector<bool> mss;

 private:
  void computeOneExtension(vector<int> *assumps, bool resultIsModel);
  bool hasAModel(vector<int> *assumps);
  vector<bool> *getModel();
};


#endif
