#include "GroundedSemanticProblemSolver.h"


using namespace std;


GroundedSemanticProblemSolver::GroundedSemanticProblemSolver(TaskType task, map<string,string> *additionalParams) : CompleteSemanticProblemSolver(task, additionalParams) {}

GroundedSemanticProblemSolver::~GroundedSemanticProblemSolver() {}


void GroundedSemanticProblemSolver::computeOneExtension(vector<int> *assumps, bool resultIsModel){
  Minisat::vec<Minisat::Lit> minisatAssumps;
  for(vector<int>::iterator itVars = assumps->begin(); itVars != assumps->end(); ++itVars) {
    minisatAssumps.push(*itVars > 0 ? Minisat::mkLit(*itVars) : ~Minisat::mkLit(*itVars));
  }
  solver.propagate(); // Perform unit propagation. Return possibly conficting clause.
  if(resultIsModel){
    result = trailToStringExtension();
  }else if(vecContains(solver.trail,minisatAssumps[0])){
    result = "YES" ;
  }else{
    result = "NO" ;
  }
}


void GroundedSemanticProblemSolver::computeAllExtensions(vector<int> *assumps){
  computeOneExtension(assumps,true);
  result = "["+ result + "]" ;
}

string GroundedSemanticProblemSolver::trailToStringExtension(){
  string res;

  bool noneFoundYet = true;
  res = "[";
  for (int i = 0; i < solver.nAssigns(); i++) {
    int var = Minisat::var(solver.trail[i]) ;
    if ((var < nbArgs) && (!Minisat::sign(solver.trail[i]))) {
      if(noneFoundYet) noneFoundYet=false; else res=res + ",";
      res = res + varMap->getName(var + 1);
    }
  }
  res = res + "]";
  return res;
}

void GroundedSemanticProblemSolver::computeProblem() {
  if(onlyOutputProblem) {
    result = "";
    displayDimacs();
    return;
  }
  result = "UNDEFINED";
  vector<int> *assumps = new vector<int>;
  switch(task) {
  case TASK_ONE_EXT:
    computeOneExtension(assumps, true);
    delete assumps;
    return;
  case TASK_ALL_EXTS:
    computeAllExtensions(assumps);
    delete assumps;
    return;
  case TASK_CRED_INF:
  case TASK_SKEP_INF:
    computeOneExtension(assumps, true);
    delete assumps;
    result = "NO";
    for (int i = 0; i < solver.nAssigns(); i++) {
      int var = Minisat::var(solver.trail[i]) ;
      if ((var < nbArgs) && (!Minisat::sign(solver.trail[i]))) {
	if(!varMap->getName(var + 1).compare(additionalParam)) {
	  result = "YES";
	  break;
	}
      }
    }
    return;
  default:
    delete assumps;
    return;
  }
}
  // case TASK_SKEP_INF:
  //   if(!additionalParam.compare("")) return;
  //   assumps->push_back(-(varMap->getVar(additionalParam)));
  //   computeOneExtension(assumps, false);
  //   if(!result.compare("YES")) result = "NO";
  //   else if (!result.compare("NO")) result = "YES";
  //   delete assumps;
  //   return;
