#ifndef __PREFERRED_SEMANTIC_PROBLEM_SOLVER_H__
#define __PREFERRED_SEMANTIC_PROBLEM_SOLVER_H__

#include "CoMSSBasedSolver.h"

#include "core/Solver.h"
#include "core/SolverTypes.h"


#define MINISAT_LBOOL_TRUE (Minisat::lbool((uint8_t)0))
#define MINISAT_LBOOL_FALSE (Minisat::lbool((uint8_t)1))
#define MINISAT_LBOOL_UNDEF (Minisat::lbool((uint8_t)2))


using namespace std;


/** 
 * \class PreferredSemanticProblemSolver
 * \brief a minisat based solver for complete semantic tasks
 */
class PreferredSemanticProblemSolver : public CoMSSBasedSolver {
  
 public :
  
  /**
   * \fn PreferredSemanticProblemSolver(TaskType,string)
   * \brief initialize the problem solver given a task
   * \param task : the task (SE, EE, DC, DS) that is asked
   * \param additionalParams the additional parameters from the command line
   */
  PreferredSemanticProblemSolver(TaskType task, map<string,string> *additionalParams);

  void initProblem(Attacks *attacks, VarMap *varMap);

  /**
   * \fn ~PreferredSemanticProblemSolver
   * \brief destructor
   */
  ~PreferredSemanticProblemSolver();

 protected:
  /** the Minisat solver instance */
  Minisat::Solver solver;

  void computeProblem();
  void computeOneMaximalExtension(vector<int> *assumps);
  void computeAllExtensions(vector<int> *assumps);
  void computeOneExtension(vector<int> *assumps, bool resultIsModel);

  bool hasAModel(vector<int> *assumps);
  vector<bool> *getModel();

};


#endif
