#include "SatBasedSolver.h"


using namespace std;


SatBasedSolver::SatBasedSolver(TaskType task, map<string,string> *additionalParams) {
  this->task = task;
  this->additionalParam = (*additionalParams)["-a"];
  this->onlyOutputProblem = additionalParams->find("-output-problem") != additionalParams->end();
}

SatBasedSolver::~SatBasedSolver() {}


void SatBasedSolver::computeProblem() {
  if(onlyOutputProblem) {
    result = "";
    displayDimacs();
    return;
  }
  result = "UNDEFINED";
  vector<int> *assumps = new vector<int>;
  switch(task) {
  case TASK_ONE_EXT:
    computeOneExtension(assumps, true);
    delete assumps;
    return;
  case TASK_ALL_EXTS:
    computeAllExtensions(assumps);
    delete assumps;
    return;
  case TASK_CRED_INF:
    if(!additionalParam.compare("")) return;
    assumps->push_back(varMap->getVar(additionalParam));
    computeOneExtension(assumps, false);
    delete assumps;
    return;
  case TASK_SKEP_INF:
    if(!additionalParam.compare("")) return;
    assumps->push_back(-(varMap->getVar(additionalParam)));
    computeOneExtension(assumps, false);
    if(!result.compare("YES")) result = "NO";
    else if (!result.compare("NO")) result = "YES";
    delete assumps;
    return;
  default:
    delete assumps;
    return;
  }
}

void SatBasedSolver::computeOneExtension(vector<int> *assumps, bool resultIsModel) {
  bool ret = hasAModel(assumps);
  if (ret) {
    vector<bool>* model = getModel();
    result = resultIsModel ? modelToStringExtension(model) : "YES";
    delete model;
  } else {
    result = "NO";
  }
}



void SatBasedSolver::computeAllExtensions(vector<int> *assumps) {
  
  result = "[";
  bool noneFoundYet = true;
  bool ret = true;
  while(ret) {
    ret = hasAModel(assumps);
    if(!ret) break;
    if(noneFoundYet) noneFoundYet=false; else result=result + ",";
    vector<bool>* model = getModel();
    result = result + modelToStringExtension(model);
    delete model;
    addBlockingClause();
  }
  result = result + "]";
}


void SatBasedSolver::displaySolution(ostream *out){
  *out << result << endl;
}


string SatBasedSolver::modelToStringExtension(vector<bool> *model) {
  string res;

  bool noneFoundYet = true;
  res = "[";
  for (int i = 0; i < nbArgs; i++) {
    if ((*model)[i]) {
      if(noneFoundYet) noneFoundYet=false; else res=res + ",";
      res = res + varMap->getName(i+1);
    }
  }
  res = res + "]";
  return res;
}
