#ifndef __SOLVER_H__
#define __SOLVER_H__


#include <iostream>

#include "Attacks.h"
#include "VarMap.h"


typedef enum {SEM_STABLE, SEM_COMPLETE, SEM_GROUNDED, SEM_PREFERRED, SEM_UNDEFINED} SemanticName;

typedef enum {TASK_ONE_EXT, TASK_ALL_EXTS, TASK_SKEP_INF, TASK_CRED_INF, TASK_UNDEFINED} TaskType;


/** 
 * \class Solver
 * \brief Interface used to build problem solvers ; provides high-level solving methods. Each class that implements this interface must be dediacated to a single precise problem, or must require informations about the problem to compute using its constructor
 */
class Solver {
  
 public:
  /**
   * \fn initProblem(Attacks*,VarMap*)
   * \brief initialize a problem given attack relations and variable mappings
   * \param attacks : the attacks relation
   * \param varMap : the variable mappings (name <-> solverVariable)
   */
  virtual void initProblem(Attacks *attacks, VarMap *varMap) = 0;

  /**
   * \fn computeProblem
   * \brief launch the problem computation process
   */
  virtual void computeProblem() = 0;

  /**
   * \fn displaySolution(ostream*)
   * \brief displays the solution of the problem to an output stream ; the printed informations are the ones required by ICCMA argumentation competition
   */
  virtual void displaySolution(ostream *out) = 0;

  /**
   * \fn ~Solver()
   * \brief destructor
   */
  virtual ~Solver() {}
};

#endif
