#ifndef __STABLE_SEMANTIC_PROBLEM_SOLVER_H__
#define __STABLE_SEMANTIC_PROBLEM_SOLVER_H__


#include "DefaultSatBasedSolver.h"


using namespace std;


/** 
 * \class StableSemanticProblemSolver
 * \brief a minisat based solver for stable semantic tasks
 */
class StableSemanticProblemSolver : public DefaultSatBasedSolver {
  
 public :
  
  /**
   * \fn StableSemanticProblemSolver(TaskType,string)
   * \brief initialize the problem solver given a task
   * \param task : the task (SE, EE, DC, DS) that is asked
   * \param additionalParams the additional parameters from the command line
   */
  StableSemanticProblemSolver(TaskType task, map<string,string> *additionalParams);

  /**
   * \fn StableSemanticProblemSolver::initProblem(Attacks*,VarMap*)
   * \brief initialize a problem given attack relations and variable mappings
   * \param attacks : the attacks relation
   * \param varMap : the variable mappings (name <-> solverVariable)
   */
  void initProblem(Attacks *attacks, VarMap *varMap);

  /**
   * \fn ~StableSemanticProblemSolver
   * \brief destructor
   */
  ~StableSemanticProblemSolver();

};


#endif
